﻿using System;
using System.Diagnostics;
using System.Globalization;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.IO;
using System.Linq;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Forms;
using System.Windows.Threading;
using TSLab.DataSource;
using Button = System.Windows.Controls.Button;
using MessageBox = System.Windows.MessageBox;

namespace GuiBarBinConverter
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private readonly ObservableCollection<TickerInfo> _tickers = new ObservableCollection<TickerInfo>();

        public MainWindow()
        {
            InitializeComponent();

            cbxTickers.ItemsSource = _tickers;
        }

        private void btnBrowseBin_Click(object sender, RoutedEventArgs e)
        {
            var dialog = new FolderBrowserDialog();
            dialog.SelectedPath = tbxBinPath.Text;
            if (dialog.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                tbxBinPath.Text = dialog.SelectedPath;
        }

        private void btnLoadBinTickers_Click(object sender, RoutedEventArgs e)
        {
            //string strMsg;
        	
        	var tickerInfos = new List<TickerInfo>();
            _tickers.Clear();

            // Проверим ввода папки
            if (!Directory.Exists(tbxBinPath.Text))
            {
                MessageBox.Show("Указана несуществующая папка.", "Ошибка!", MessageBoxButton.OK, MessageBoxImage.Error);
                return;
            }
                

            var files = Directory.GetFiles(tbxBinPath.Text, "*.bin");
            
            foreach (var path in files)
            {
                // Читаем версию бинарника
                //MessageBox.Show("Читаем версию бинарника.", "Information.", MessageBoxButton.OK, MessageBoxImage.Information);
                //var binVer = BinaryCache<TradeInfo>.ReadVersion(path);
                var binVer = BinaryCache<DataBar>.ReadVersion(path);

                // Получаем тикер из имени файла, и если тикер новый, добавляем имя тикера в общий список
                // Проверяем версии бинарников, если в двух версиях, то указываем обе версии.
                var ticker = Path.GetFileNameWithoutExtension(path);
                //var ticker = name;
                
                // Ищем в списке тикеров только что прочитанные, если он уже есть, 
                // и версия бинарника уже такая есть, то переходим к след файлу.
                var et = tickerInfos.Find(info => info.Ticker == ticker);
                if (et != null && et.Versions.Contains(binVer))
                    continue;

                // Если тикер уже есть но версия файла прочитана новая.
                if (et != null)
                    et.Versions.Add(binVer);

                // Если тикера еще нет, добавим новый тикер в список.
                if (et == null)
                    tickerInfos.Add(new TickerInfo(ticker, new List<int>(){binVer}));
            }

            tickerInfos.Sort((info1, info2) => string.Compare(info1.Ticker, info2.Ticker, StringComparison.InvariantCultureIgnoreCase));
            tickerInfos.ForEach(_tickers.Add);
        }

        private void btnSaveBin2Txt_Click(object sender, RoutedEventArgs e)
        {
            // Получим выбранный тикер, если ничего не выбрали то выходим.
            var tickerInfo = cbxTickers.SelectedItem as TickerInfo;
            if (tickerInfo == null)
                return;

            // Выключим кнопку записи в файл чтобы сто раз не жмакали
            ((Button) sender).IsEnabled = false;

            var sortedDictionary = new SortedDictionary<DateTime, string>();
            var binPath = tbxBinPath.Text;

            // Читаем все имена файлов содержащие выбранный тикер
            var files = Directory.GetFiles(binPath, tickerInfo.Ticker + "*.bin").Select(Path.GetFileName).ToArray();

            // Инициализируем прогресс бар
            prgbBin2Txt.Minimum = 0;
            prgbBin2Txt.Maximum = sortedDictionary.Count - 1;
            prgbBin2Txt.Value = 0;
            prgbBin2Txt.IsIndeterminate = false;


            // теперь берем даты поочереди и переводим в текст

            var cache = new BinaryCache<DataBar>();
            var streamW = new StreamWriter(@"{0}\{1}.{2}".Put(binPath, tickerInfo.Ticker, "txt"), false);

            var txtTemplate = "{0:yyyyMMdd,HHmmss},{1},{2},{3},{4},{5}";
            streamW.WriteLine(txtTemplate.Put("Date,Time", "Open", "High", "Low", "Close", "Volume"));

            var path = @"{0}\{1}".Put(binPath, tickerInfo.Ticker + ".bin");
            var version = BinaryCache<DataBar>.ReadVersion(path);
            if (version == 0)
            	throw new Exception("Не удалось получить версию формата файла.");

            var bars = cache.LoadCached(path, version);

   			NumberFormatInfo nfi = new CultureInfo( "en-US", false ).NumberFormat;
            bars.ForEach( bar =>
                            streamW.WriteLine(txtTemplate.Put(bar.Date, 
   			                                                 bar.Open.ToString(nfi), 
   			                                                 bar.High.ToString(nfi), 
   			                                                 bar.Low.ToString(nfi), 
   			                                                 bar.Close.ToString(nfi), 
   			                                                 bar.Volume)));

            // Обновляем прогресс бар
            this.Dispatcher.BeginInvoke(new Action(() => prgbBin2Txt.Value += 1), DispatcherPriority.Normal);

            // Скидываем на диск и закрываем потоки.
            streamW.Flush();
            streamW.Close();

            // Включим кнопку записи в файл
            this.Dispatcher.BeginInvoke(new Action(() => ((Button) sender).IsEnabled = true), DispatcherPriority.Normal);
        }

        private void btnBrowseTxt_Click(object sender, RoutedEventArgs e)
        {
            var dialog = new OpenFileDialog() {Multiselect = false};
            if (dialog.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                tbxTxtPath.Text = dialog.FileName;
             }
        }

        private void btnSaveTxt2Bin_Click(object sender, RoutedEventArgs e)
        {
            // Выключим кнопку записи в файл чтобы сто раз не жмакали
            ((Button) sender).IsEnabled = false;

            // Читаем ввод пользователя
            var binVersion = int.Parse(tbxBinVer.Text);
            var binPrefix = tbxBinPrefix.Text;
            var filePath = tbxTxtPath.Text;

            // Проверим ввода файла
            if (!File.Exists(filePath))
            {
                MessageBox.Show("Указан несуществующий файл.", "Ошибка!", MessageBoxButton.OK, MessageBoxImage.Error);
                return;
            }
            
            var dirPath = Path.GetDirectoryName(filePath);

            // Инициализируем прогресс бар
            prgbTxt2Bin.IsIndeterminate = true;


            var cache = new BinaryCache<DataBar>();
            var sr = new StreamReader(filePath);

            var bars = new List<DataBar>();
            //var currDate = DateTime.MinValue;

            // Пропустим первую строчку ибо в ней заголовки
            sr.ReadLine();


            while (!sr.EndOfStream) {
            	var line = sr.ReadLine();
                //Date,Time,Open,High,Low,Close,Volume
                var parts = line.Split(new char[] { ',' }, StringSplitOptions.RemoveEmptyEntries);
                var ic = CultureInfo.InvariantCulture;
                NumberFormatInfo nfi = new CultureInfo( "en-US", false ).NumberFormat;
                var dt = DateTime.ParseExact(parts[0], "yyyyMMdd", ic);
                var tm = DateTime.ParseExact(parts[1], "HHmmss", ic);
                TimeSpan ts = new TimeSpan(tm.Hour, tm.Minute, tm.Second  );
                dt = dt.Add(ts );
                var op = double.Parse(parts[2], nfi);
                var hi = double.Parse(parts[3], nfi);
                var lo = double.Parse(parts[4], nfi);
                var cl = double.Parse(parts[5], nfi);
                var vl = double.Parse(parts[6], ic);
                var bar = new DataBar(dt, op, hi, lo, cl, vl);
                bars.Add(bar);
            } //while

            //var path = Path.Combine(dirPath, @"{0}.bin".Put(binPrefix, currDate));
            var path = Path.Combine(dirPath, @"{0}.bin".Put(binPrefix) );
            cache.SaveCached(path, bars, binVersion);

			sr.Close();

			// Включим кнопку записи в файл
			this.Dispatcher.BeginInvoke(new Action(() => ((Button) sender).IsEnabled = true), DispatcherPriority.Normal);

			// Обновляем прогресс бар
			this.Dispatcher.BeginInvoke(new Action(() =>
                                       {
                                       	prgbTxt2Bin.IsIndeterminate = false;
                                       	prgbTxt2Bin.Value = 0;
                                       }), DispatcherPriority.Normal);
          
        }
    }
}

